@@ -3,7 +3,7 @@
 use crate::{PublicKey, Result, KEY_SIZE};
 
 #[cfg(feature = "secp256k1")]
-use crate::Error;
+use crate::{Error, XPrv};
 
 /// Bytes which represent a private key.
 pub type PrivateKeyBytes = [u8; KEY_SIZE];
@@ -19,11 +19,12 @@ pub trait PrivateKey: Sized {
     /// Serialize this key as bytes.
     fn to_bytes(&self) -> PrivateKeyBytes;
 
-    /// Derive a child key from a parent key and the left-half of the output
-    /// of HMAC-SHA512 (where `left_half` is referred to as "I sub L" in BIP32)
-    fn derive_child(&self, left_half: PrivateKeyBytes) -> Result<Self>;
+    /// Derive a child key from a parent key and the a provided tweak value,
+    /// i.e. where `other` is referred to as "I sub L" in BIP32 and sourced
+    /// from the left half of the HMAC-SHA-512 output.
+    fn derive_child(&self, other: PrivateKeyBytes) -> Result<Self>;
 
-    /// Get the [`PublicKey`] that corresponds to this private key.
+    /// Get the [`Self::PublicKey`] that corresponds to this private key.
     fn public_key(&self) -> Self::PublicKey;
 }
 
@@ -40,8 +41,8 @@ impl PrivateKey for k256::SecretKey {
         k256::SecretKey::to_bytes(self).into()
     }
 
-    fn derive_child(&self, left_half: PrivateKeyBytes) -> Result<Self> {
-        let child_scalar = k256::NonZeroScalar::from_repr(left_half.into()).ok_or(Error::Crypto)?;
+    fn derive_child(&self, other: PrivateKeyBytes) -> Result<Self> {
+        let child_scalar = k256::NonZeroScalar::from_repr(other.into()).ok_or(Error::Crypto)?;
         let derived_scalar = self.to_secret_scalar().as_ref() + child_scalar.as_ref();
 
         k256::NonZeroScalar::new(derived_scalar)
@@ -67,14 +68,30 @@ impl PrivateKey for k256::ecdsa::SigningKey {
         k256::ecdsa::SigningKey::to_bytes(self).into()
     }
 
-    fn derive_child(&self, left_half: PrivateKeyBytes) -> Result<Self> {
+    fn derive_child(&self, other: PrivateKeyBytes) -> Result<Self> {
         k256::SecretKey::from(self)
-            .derive_child(left_half)
+            .derive_child(other)
             .map(Into::into)
     }
 
     fn public_key(&self) -> Self::PublicKey {
-        self.verify_key()
+        self.verifying_key()
+    }
+}
+
+#[cfg(feature = "secp256k1")]
+#[cfg_attr(docsrs, doc(cfg(feature = "secp256k1")))]
+impl From<XPrv> for k256::ecdsa::SigningKey {
+    fn from(xprv: XPrv) -> k256::ecdsa::SigningKey {
+        k256::ecdsa::SigningKey::from(&xprv)
+    }
+}
+
+#[cfg(feature = "secp256k1")]
+#[cfg_attr(docsrs, doc(cfg(feature = "secp256k1")))]
+impl From<&XPrv> for k256::ecdsa::SigningKey {
+    fn from(xprv: &XPrv) -> k256::ecdsa::SigningKey {
+        xprv.private_key().clone()
     }
 }
 
@@ -91,9 +108,9 @@ impl PrivateKey for secp256k1_ffi::SecretKey {
         *self.as_ref()
     }
 
-    fn derive_child(&self, left_half: PrivateKeyBytes) -> Result<Self> {
+    fn derive_child(&self, other: PrivateKeyBytes) -> Result<Self> {
         let mut child = *self;
-        child.add_assign(&left_half)?;
+        child.add_assign(&other)?;
         Ok(child)
     }
 
@@ -105,7 +122,7 @@ impl PrivateKey for secp256k1_ffi::SecretKey {
 }
 
 /// `secp256k1-ffi` smoke tests
-#[cfg(all(test, feature = "secp256k1-ffi"))]
+#[cfg(all(test, feature = "bip39", feature = "secp256k1-ffi"))]
 mod tests {
     use hex_literal::hex;
 
@@ -119,7 +136,7 @@ mod tests {
         );
 
         let path = "m/0/2147483647'/1/2147483646'/2";
-        let xprv = XPrv::derive_child_from_seed(&seed, &path.parse().unwrap()).unwrap();
+        let xprv = XPrv::derive_from_path(&seed, &path.parse().unwrap()).unwrap();
 
         assert_eq!(
             xprv,
